// Quick Barrier Constructor
// by Gigan
//
// This script adds the barrier construction capability to player.
// The player which changed to a engineer copes with invasion of enemy ground troops.
// The player can specify object direction and can put it in order continuously on the map.
//
// Execution
// handle = [ "Land_HBarrier5", 10 ] execVM "qbc.sqf";
//
// 1st argument is object class name for putting.
// 2nd argument is amount of objects for putting continuously.
//
// Supported objects which show to suitable marker size.
// H-barrier
// H-barrier (short)
// H-barrier cube
// H-barrier (large)
//
// Mouse and Key operation.
// Left Mouse Click : Put a Object
// Left Mouse Drag  : Rotate a Object
// Left Ctrl        : Undo
// Esc              : Quit Construction
//
// Known bug
// The terminated that depends besides Esc key might occur.
//


_objectname = _this select 0;
_num = _this select 1;

MapClicked = false;
MapClickedPosX = 0;
MapClickedPosY = 0;

MouseMovingEvent = [controlNull,0,0,false];
MouseHoldingEvent = [controlNull,0,0,false];
MouseDownEvent = [controlNull,-1,0,0,false,false,false];
MouseUpEvent = [controlNull,-1,0,0,false,false,false];
KeyDownEventMap = [controlNull,-1,false,false,false];
KeyDownEventDisp = [controlNull,-1,false,false,false];

PutNum = 0;

_cnt = 0;
_downkey = -1;
_mousepos_map = [0,0];
_mousepos_world = [0,0,0];
_setdone = false;
_rotatedone = false;
_icondir = 0;
_mapshow = false;
_esc = false;
_sizew = 0;
_sizeh = 0;
_offsetx = 0;
_offsety = 0;
_prename = "barrier" + format["%1",time];
_objects = [];
_markers = [];

// Help.
hint format["Single click on the map.\n0/%1\n\nMouse Drag : Rotate Object\nLeft Ctrl : Undo\nEsc : Quit\n\n%2", _num];

// Adding class names, marker size can be made to correspond.
// When object center has shifted from object rotation axis, adjust offset.
switch (_objectname) do
{
	case ( "Land_HBarrier1" ):
	{ _sizew = 0.8; _sizeh = 0.8; _offsetx = 0; _offsety = 0 };
	
	case ( "Land_HBarrier3" ):
	{ _sizew = 1.7; _sizeh = 0.8; _offsetx = -1.08; _offsety = 0 };
	
	case ( "Land_HBarrier5" ):
	{ _sizew = 2.8; _sizeh = 0.8; _offsetx = -2.12; _offsety = -0.4 };
	
	case ( "Land_HBarrier_large" ):
	{ _sizew = 4.2; _sizeh = 1.1; _offsetx = 0; _offsety = 0 };
	
	default
	{ _sizew = 2; _sizeh = 1; _offsetx = 0; _offsety = 0 };
};

onMapSingleClick "MapClickedPosX = _pos select 0; MapClickedPosY =_pos select 1; MapClicked = true";

(findDisplay 12 displayCtrl 51) ctrlSetEventHandler ["MouseMoving", "MouseMovingEvent = _this"];
(findDisplay 12 displayCtrl 51) ctrlSetEventHandler ["MouseHolding", "MouseHoldingEvent = _this"];
(findDisplay 12 displayCtrl 51) ctrlSetEventHandler ["MouseButtonDown", "MouseDownEvent = _this"];
(findDisplay 12 displayCtrl 51) ctrlSetEventHandler ["MouseButtonUp", "MouseUpEvent = _this"];
(findDisplay 12 displayCtrl 51) ctrlSetEventHandler ["KeyDown", "KeyDownEventMap = _this"];
(findDisplay 46) displaySetEventHandler ["KeyDown", "KeyDownEventDisp = _this"];

createMarkerLocal[ "assumeicon", [ MapClickedPosX, MapClickedPosY ] ];
"assumeicon" setMarkerShapeLocal "RECTANGLE";
"assumeicon" setMarkerBrushLocal "SOLID";
"assumeicon" setMarkerColorLocal "ColorOrange";
"assumeicon" setMarkerAlphaLocal 0.5;
"assumeicon" setMarkerSizeLocal [_sizew, _sizeh];

while { (_cnt < _num) and !(_esc) } do
{
	MouseDownEvent = [controlNull,-1,0,0,false,false,false];
	_setdone = false;
	
	while { !(_setdone) and !(_esc) } do
	{
		_downkey = KeyDownEventDisp select 1;
		
		if ( _mapshow and (format["%1",(MouseMovingEvent select 0)] == "NO CONTROL") and (format["%1",(MouseHoldingEvent select 0)] == "NO CONTROL") ) then
		{
			if ( (_downkey != 50) and (_downkey != 83) and (_downkey != 156) and (_downkey != 20) ) then { _esc = true };
			_mapshow = false;
		};
		
		if ( !(_mapshow) ) then
		{
			if ( (format["%1",(MouseMovingEvent select 0)] != "NO CONTROL") or (format["%1",(MouseHoldingEvent select 0)] != "NO CONTROL") ) then
			{
				_mapshow = true;
			};
			
			KeyDownEventDisp = [controlNull,-1,false,false,false];
			
			sleep 0.1;
		};
		
		if ( format["%1",(MouseMovingEvent select 0)] != "NO CONTROL" ) then
		{
			_mousepos_map = [ MouseMovingEvent select 1, MouseMovingEvent select 2 ];
			_mousepos_world = (findDisplay 12 displayCtrl 51) posScreenToWorld _mousepos_map;
			"assumeicon" setMarkerPosLocal _mousepos_world;
		};
		
		if ( (MouseDownEvent select 1) == 0 ) then
		{
			_setdone = true;
			
			_rotatedone = false;
			MouseUpEvent = [controlNull,-1,0,0,false,false,false];
			MouseMovingEvent set [1,_mousepos_map select 0];
			MouseMovingEvent set [2,_mousepos_map select 1];
			
			while {!(_rotatedone)} do
			{
				if ((MouseUpEvent select 1) == 0) then { _rotatedone = true };
				
				if ( _mousepos_map distance [ MouseMovingEvent select 1, MouseMovingEvent select 2 ] > 0.02) then { _icondir = (((_mousepos_map select 1)-(MouseMovingEvent select 2)) atan2 ((_mousepos_map  select 0)-(MouseMovingEvent select 1))) - 90 };
				"assumeicon" setMarkerDirLocal _icondir;
				
				sleep 0.01;
			};
		};
		
		if ( (KeyDownEventMap select 1 == 29) and (_cnt > 0) ) then
		{
			sleep 0.1;
			
			deletevehicle (_objects select 0);
			deletemarkerlocal (_markers select 0);
			
			_objects = _objects - [_objects select 0];
			_markers = _markers - [_markers select 0];
			
			_cnt = _cnt - 1;
			
			hint format["Undo.\n%1/%2",_cnt,_num];
		};
		
		MouseMovingEvent = [controlNull,0,0,false];
		MouseHoldingEvent = [controlNull,0,0,false];
		KeyDownEventMap = [controlNull,-1,false,false,false];
		
		sleep 0.01;
	};
	
	if (_setdone) then
	{
		_obj = _objectname createVehicle [ (_mousepos_world select 0) + _offsetx, (_mousepos_world select 1) + _offsety, 0 ];
		_obj setpos [ (_mousepos_world select 0) + _offsetx, (_mousepos_world select 1) + _offsety, 0 ];
		_obj setdir _icondir;
		
		_name = _prename + format["_%1",_cnt];
		_marker = createMarkerLocal[ _name, [ _mousepos_world select 0, _mousepos_world select 1 ] ];
		_name setMarkerShapeLocal "RECTANGLE";
		_name setMarkerBrushLocal "SOLID";
		_name setMarkerColorLocal "ColorOrange";
		_name setMarkerDirLocal _icondir;
		_name setMarkerAlphaLocal 1.0;
		_name setMarkerSizeLocal [_sizew, _sizeh];
		
		_objects = [_obj] + _objects;
		_markers = [_marker] + _markers;
		
		_cnt = _cnt + 1;
		
		hint format["Placed.\n%1/%2",_cnt,_num];
	};
};


deletemarkerlocal "assumeicon";
onMapSingleClick "";
(findDisplay 12) displayRemoveAllEventHandlers "MouseMoving";
(findDisplay 12) displayRemoveAllEventHandlers "MouseHolding";
(findDisplay 12) displayRemoveAllEventHandlers "MouseButtonDown";
(findDisplay 12) displayRemoveAllEventHandlers "MouseButtonUp";
(findDisplay 12) displayRemoveAllEventHandlers "KeyDown";
(findDisplay 46) displayRemoveAllEventHandlers "KeyDown";


if (_esc) then
{ hint "Terminated." }
else
{ hint "Completed." };

PutNum = _cnt;
